%{
#define YYERROR_VERBOSE 1
void
  yyerror (char *s);
 %}
%union {
  int intValue;
  char *stringValue;
}


%token <stringValue>T_ID
%token <intValue>T_INT
%token T_KW_INT
%token T_KW_RETURN
%token T_KW_IF
%token T_KW_ELSE
%token T_KW_WHILE

%token T_OP_EQ
%token T_OP_ASSIGN

%%

program : functionDefinition
| program functionDefinition
;

functionDefinition : T_KW_INT T_ID '(' ')' '{' statements '}'
{printf("PARSED FUNCTION %s\n",$2);}
| T_KW_INT T_ID '(' parameters ')' '{' statements '}'
{printf("PARSED FUNCTION %s\n",$2);}
| T_KW_INT T_ID  '(' ')' '{' declarations statements '}'
{printf("PARSED FUNCTION %s\n",$2);}
| T_KW_INT T_ID  '('parameters ')' '{' declarations statements '}'
{printf("PARSED FUNCTION %s\n",$2);}
;

parameters : parameter
| parameters ',' parameter
;

parameter : T_KW_INT T_ID
{printf("PARSED PARAMETER %s\n",$2);}
;

declarations : declaration
| declarations declaration
;

declaration : T_KW_INT T_ID ';'
{printf ("PARSED LOCAL VARIABLE %s\n",$2);}
;

statements : statement
| statements statement
;

statement : '{' statements '}'
| T_ID T_OP_ASSIGN expr ';'
{/*printf("ASSIGNED VALUE TO %s\n",$1);*/}
| T_KW_IF '(' boolExpr ')' statement
| T_KW_WHILE '(' boolExpr ')' statement
| T_KW_RETURN expr ';'
;

expr : term
| expr '+' term
| expr '-' term
;

term : factor
| term '*' factor
;

factor : T_INT
{}
| '(' expr ')'
| T_ID
{}
| T_ID '(' ')'
{}
| T_ID '(' exprList ')'
{}
;

exprList : expr
| exprList ',' expr
;

boolExpr : relExpr
| '!' '(' relExpr ')'
;

relExpr : expr T_OP_EQ expr
| expr '>' expr
;

%%

void
yyerror (char *s)
{
  fprintf (stderr, "Err: %s\n", s);
}

int
main (void)
{
  return yyparse ();
}
